# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This file can not be copied and/or distributed without the express
#  permission of Comet ML Inc.
# *******************************************************

from typing import IO, Any, List, Optional

from . import comet_api_client


class ExperimentAPI:
    def __init__(
        self,
        api_key: str,
        workspace: Optional[str],
        project_name: Optional[str],
    ):
        self._client = comet_api_client.get(api_key)
        self._initialize_experiment(workspace, project_name)

    @property
    def link(self) -> str:
        return self._link

    @property
    def project_link(self) -> str:
        return self._project_link

    def _initialize_experiment(
        self, workspace: Optional[str] = None, project_name: Optional[str] = None
    ) -> None:
        response = self._client.create_experiment("LLM", workspace, project_name)
        self._experiment_key = response["experimentKey"]
        self._initialize_links(response["link"])

    def _initialize_links(self, link: str) -> None:
        self._link = link
        self._project_link = link[: link.rfind("/")]

    def log_asset_with_io(self, name: str, file: IO, asset_type: str) -> None:
        self._client.log_experiment_asset_with_io(
            self._experiment_key, name=name, file=file, asset_type=asset_type
        )

    def log_parameter(self, name: str, value: Any) -> None:
        self._client.log_experiment_parameter(
            self._experiment_key, name=name, value=value
        )

    def log_metric(self, name: str, value: Any) -> None:
        self._client.log_experiment_metric(self._experiment_key, name=name, value=value)

    def log_tags(self, tags: List[str]) -> None:
        self._client.log_experiment_tags(self._experiment_key, tags=tags)



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This file can not be copied and/or distributed without the express
#  permission of Comet ML Inc.
# *******************************************************

import logging

from . import logs_registry

LOGGER = logging.getLogger(__name__)


class Summary:
    def __init__(self) -> None:
        self._registry = logs_registry.LogsRegistry()

    def add_log(self, project_url: str) -> None:
        if self._registry.empty():
            LOGGER.info("Prompt logged to %s", project_url)

        self._registry.register_log(project_url)

    def print(self) -> None:
        registry_items = self._registry.as_dict().items()

        for project, logs_amount in registry_items:
            LOGGER.info("%d prompts logged to %s", logs_amount, project)



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This file can not be copied and/or distributed without the express
#  permission of Comet ML Inc.
# *******************************************************

import functools
import logging
from types import ModuleType
from typing import TYPE_CHECKING, Optional


def _muted_import_comet_ml() -> ModuleType:
    try:
        logging.disable(logging.CRITICAL)
        import comet_ml

        return comet_ml  # type: ignore
    finally:
        logging.disable(0)


comet_ml = _muted_import_comet_ml()

if TYPE_CHECKING:
    import comet_ml.config as comet_ml_config


@functools.lru_cache(maxsize=1)
def _comet_ml_config() -> "comet_ml_config.Config":
    return comet_ml.get_config()


def workspace() -> Optional[str]:
    return _comet_ml_config()["comet.workspace"]  # type: ignore


def project_name() -> Optional[str]:
    return _comet_ml_config()["comet.project_name"]  # type: ignore


def comet_url() -> str:
    return comet_ml.get_backend_address(_comet_ml_config())  # type: ignore


def api_key() -> Optional[str]:
    comet_ml_config = _comet_ml_config()
    api_key = comet_ml.get_api_key(None, comet_ml_config)
    return api_key  # type: ignore



# -*- coding: utf-8 -*-
# *******************************************************
#   ____                     _               _
#  / ___|___  _ __ ___   ___| |_   _ __ ___ | |
# | |   / _ \| '_ ` _ \ / _ \ __| | '_ ` _ \| |
# | |__| (_) | | | | | |  __/ |_ _| | | | | | |
#  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
#
#  Sign up for free at https://www.comet.com
#  Copyright (C) 2015-2023 Comet ML INC
#  This file can not be copied and/or distributed without the express
#  permission of Comet ML Inc.
# *******************************************************


import functools
from typing import Any, Callable

import requests  # type: ignore

from .. import exceptions


def wrap(func: Callable) -> Callable:
    @functools.wraps(func)
    def wrapper(*args, **kwargs) -> Any:  # type: ignore
        try:
            return func(*args, **kwargs)
        except requests.RequestException as exception:
            raise exceptions.CometLLMException() from exception

    return wrapper
